import argparse
import json
import pickle

import pandas as pd
import shap
import xgboost as xgb
from catboost import CatBoostClassifier
from sklearn.ensemble import RandomForestClassifier

from dscontexai.dsc_grammar import analyze_shap_values


def get_shap(model, X: pd.DataFrame, index: int) -> shap.Explainer:
    if isinstance(model, xgb.XGBClassifier):
        explainer = shap.Explainer(model, X, model_output="probability")
        shap_values = explainer(X.iloc[index])
    else:
        explainer = shap.KernelExplainer(model.predict_proba, shap.sample(X, 1000))
        shap_values = explainer(X.iloc[index])
        if shap_values.values.shape[1] == 2:
            shap_values.values = shap_values.values[:, 1]
            shap_values.base_values = shap_values.base_values[:, 1][0]

    return shap_values


def diabetes_case() -> None:
    """
    Load the diabetes model and data, and generate a report for a specific instance.
    """
    df = pd.read_csv("./kaggle_data/processed_data.csv")
    X = df.drop(columns=["diabetes"])
    y = df["diabetes"]

    with open("./models/diabetes_model.pkl", "rb") as f:
        model = pickle.load(f)

    # Expert descriptions for values below, within, and above optimal ranges
    # read from json file
    with open("configurations/diabetes.json") as f:
        data = json.load(f)

    optimal_values = data["optimal_values"]
    below_optimal_descriptions = data["descriptions"]["below_optimal"]
    optimal_descriptions = data["descriptions"]["optimal"]
    above_optimal_descriptions = data["descriptions"]["above_optimal"]

    transformations = data["transformation"]

    feature_names = data["feature_names"]
    target1 = data["target1"]
    target2 = data["target2"]
    supporting = data["supporting"]
    index = data["index"]

    explainer = shap.Explainer(model, X, model_output="probability")
    shap_values_array = explainer(X.iloc[index])

    analyze_shap_values(
        shap_values_array,
        index,
        target1,
        target2,
        feature_names,
        supporting,
        optimal_values,
        below_optimal_descriptions,
        optimal_descriptions,
        above_optimal_descriptions,
        transformations,
    )


def general(model_path, dataset_path, config, idx):
    """
    Load the insurance model and data, and generate a report for a specific instance.
    """

    # Load prediction model
    with open(model_path, "rb") as f:
        model = pickle.load(f)

    # Expert descriptions for values below, within, and above optimal ranges
    # read from json file
    with open(config) as f:
        data = json.load(f)

    optimal_values = data["optimal_values"]
    below_optimal_descriptions = data["descriptions"]["below_optimal"]
    optimal_descriptions = data["descriptions"]["optimal"]
    above_optimal_descriptions = data["descriptions"]["above_optimal"]

    transformations = data["transformation"]

    feature_names = data["feature_names"]
    target1 = data["target1"]
    target2 = data["target2"]
    supporting = data["supporting"]

    df = pd.read_csv(dataset_path)
    X = df.drop(columns=[target1])
    y = df[target1]

    shap_values_array = get_shap(model, X, idx)

    analyze_shap_values(
        shap_values_array,
        idx,
        target1,
        target2,
        feature_names,
        supporting,
        optimal_values,
        below_optimal_descriptions,
        optimal_descriptions,
        above_optimal_descriptions,
        transformations,
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Automatic Report Generator for xAI")

    parser.add_argument("--instance", help="Instance index", action="store", default=1, type=int)
    parser.add_argument("--model", help="Model path", action="store", default=None)
    parser.add_argument("--dataset", help="Data path", action="store", default=None)
    parser.add_argument("--config", help="Config path", action="store", default=None)
    parser.add_argument("--index", help="Instance index", action="store", default=1, type=int)
    parser.add_argument(
        "--example",
        help="Shows working example of the library",
        action="store_true",
        required=False,
    )

    args = parser.parse_args()

    if args.index < 0:
        print("Index has to be positive!")
        exit()

    if args.example:
        model_path = "./models/diabetes_model.pkl"
        df = pd.read_csv("./kaggle_data/processed_data.csv")
        diabetes_case()
    else:
        model_path = args.model
        data = args.dataset
        config = args.config
        general(model_path, data, config, args.index)



import os
import shutil

import matplotlib.pyplot as plt
import numpy as np
import pkg_resources
import shap
from pylatex import Document, Figure, Itemize, NoEscape, Package, Section, Subsection
from pylatex.utils import bold

from dscontexai.dsc_ploting import plot_shap

plt.rcParams["text.usetex"] = True


def extract_neur_sty(target_directory):
    # Ensure the target directory exists
    if not os.path.exists(target_directory):
        os.makedirs(target_directory)

    # Get the path to neur.sty within the package
    sty_path = pkg_resources.resource_filename("dscontexai", "latex/neurips_2023.sty")

    # Define the target path where the file should be copied
    target_path = os.path.join(target_directory, "neurips_2023.sty")

    # Copy the file to the target directory
    shutil.copyfile(sty_path, target_path)

    return target_path


extracted_path = extract_neur_sty("./latex/")


def fixing_variable_names(variables: list) -> list:
    """
    Fix the variable names by removing underscores and capitalizing the first letter of each word.

    Parameters
    ----------
    variables : list
        List of variable names.

    Returns
    -------
    list
        List of fixed variable names.
    """

    fixed_names = []
    for var in variables:
        # Remove underscores and capitalize the first letter of each word
        fixed_name = var.replace("_", " ")
        # Capitalize the first letter of the first word
        fixed_name = fixed_name[0].upper() + fixed_name[1:]
        fixed_names.append(fixed_name)

    return fixed_names


def fix_feature_values(feature_values: list) -> list:
    """
    Fix the feature values by rounding them to 2 decimal places.

    Parameters
    ----------
    feature_values : list
        List of feature values.

    Returns
    -------
    list
        List of fixed feature values.
    """

    fixed_values = []

    for value in feature_values:
        value = int(value) if value.is_integer() else value
        fixed_values.append(value)

    return fixed_values


def check_values(
    data: list,
    optimal_values: list,
    below_optimal_descriptions: list,
    optimal_descriptions: list,
    above_optimal_descriptions: list,
) -> list:
    """
    Check the values of the instance against the optimal values.

    Parameters
    ----------
    data : list
        List of feature values.
    optimal_values : list
        List of optimal value ranges.
    below_optimal_descriptions : list
        List of descriptions for values below the optimal range.
    optimal_descriptions : list
        List of descriptions for values within the optimal range.

    Returns
    -------
    list
        List of descriptions for each feature value.
    """

    results = []
    for (low, high), value, below_desc, optimal_desc, above_desc in zip(
        optimal_values,
        data,
        below_optimal_descriptions,
        optimal_descriptions,
        above_optimal_descriptions,
    ):
        if value < low:
            results.append(below_desc)
        elif low <= value <= high:
            results.append(optimal_desc)
        else:
            results.append(above_desc)

    return results


def transform_instance_values(instance_values: list, transformations: dict) -> list:
    """
    Transform the instance values based on the transformations.

    Parameters
    ----------
    instance_values : list
        List of instance values.
    transformations : dict
        Dictionary of transformations for the instance values.

    Returns
    -------
    list
        List of transformed instance values.
    """

    new_instance_values = []
    for index in range(len(transformations.values())):
        item = transformations.get(str(index))
        if len(item) == 0:
            new_instance_values.append(instance_values[index])
            continue
        value = instance_values[index]
        if isinstance(value, str):
            new_instance_values.append(instance_values[index])
        else:
            new_instance_values.append(item[instance_values[index]])
    return new_instance_values


def analyze_shap_values(
    shap_values_array: shap.Explanation,
    index: int,
    target1: str,
    target2: str,
    feature_names: list,
    supporting,
    optimal_values: list,
    below_optimal_descriptions: list,
    optimal_descriptions: list,
    above_optimal_descriptions: list,
    transformations: dict,
) -> None:
    """
    Analyze the SHAP values and create a report.

    Parameters
    ----------
    shap_values_array : list
        SHAP values for the instance.
    feature_names : list
        List of feature names.
    target1 : str
        Target class name.
    index : int
        Index of the instance.
    target2 : str
        Target class name.
    supporting : list
        Supporting words.
    optimal_values : list
        List of optimal value ranges.
    below_optimal_descriptions : list
        List of descriptions for values below the optimal range.
    optimal_descriptions : list
        List of descriptions for values within the optimal range.
    above_optimal_descriptions : list
        List of descriptions for values above the optimal range.
    transformations : list
        List of transformations for the instance values.

    Returns
    -------
    None
    """

    shap_values = shap_values_array.values
    starting_value = shap_values_array.base_values
    if type(starting_value) == np.ndarray:
        starting_value = starting_value[0]
    feature_values = shap_values_array.data

    # Sort the shap_values by their magnitude
    indices = np.argsort(np.abs(shap_values))[::-1]  # Descending order

    feature_names = fixing_variable_names(feature_names)
    feature_values = fix_feature_values(feature_values)
    instance_values = transform_instance_values(feature_values, transformations)

    # Create sorted arrays
    sorted_shap_values = shap_values[indices]
    # Round to 2 decimal places
    sorted_shap_values = sorted_shap_values.tolist()
    sorted_shap_values = [round(value, 2) for value in sorted_shap_values]
    sorted_shap_values = np.array(sorted_shap_values)
    sorted_feature_names = [feature_names[i] for i in indices]

    # Calculate the change array
    change = np.cumsum(sorted_shap_values) + round(starting_value, 2)

    if change[-1] > 0.5:
        status = supporting[0] + " " + supporting[1] if len(supporting) > 1 else ""
    else:
        status = f"{supporting[0]} not {supporting[1] if len(supporting) > 1 else ''}"

    if abs(change[-1] - 0.5) > 0.3:
        confidence = f"""The model is {bold("confident")} in its prediction, as the probability that the person {status} {target1} {bold("is close to 1")}."""
    else:
        confidence = f"""The model is not {bold("confident")} in its prediction, as the probability that the person {status} {target1} {bold("is close to 0.5")}."""

    output = f"""The inital prediction value equals average prediction {round(starting_value, 2)}. The feature that influences the final prediction the most for this example is {bold(sorted_feature_names[0])} with an importance of {bold(str(round(sorted_shap_values[0],2)))}, which changes the initial value to {bold(str(round(change[0],2)))}. 
    The second most important feature for this example is {bold(sorted_feature_names[1])} with a value of {bold(str(round(sorted_shap_values[1],2)))}. Now, the prediction amounts to {bold(str(round(change[1],2)))}. 
    The third most important feature is {bold(sorted_feature_names[2])} with an importance of {bold(str(round(sorted_shap_values[2],2)))}. This changes the prediction to {bold(str(round(change[2],2)))}.
    Other features together contribute {bold(str(round((change[len(change)-1]-change[2]),2)))}.
    The final prediction is {bold(str(round(change[len(change)-1],2)))}. Because the prediction is {bold("greater") if change[len(change)-1] > 0.5 else bold("less")} than 0.5, the model predicts that the {bold(target2.lower())} {bold(status)} {bold(target1)}. 
    {confidence}\n\n"""

    # Create the data for plotting
    feature_values_named = [
        f"{name} = {value}" for name, value in zip(feature_names, instance_values)
    ]

    # Sort features by absolute SHAP value
    sorted_indices = sorted(
        range(len(shap_values)), key=lambda k: abs(shap_values[k]), reverse=True
    )
    sorted_features = [feature_values_named[i] for i in sorted_indices]
    sorted_shap_values = [shap_values[i] for i in sorted_indices]

    # Set up the plot
    plot_shap(sorted_shap_values, sorted_features, "./plots/shap.pdf")

    doc = Document(documentclass="article", document_options="a4paper", page_numbers=False)
    nips_style = pkg_resources.resource_filename("dscontexai", "latex/neurips_2023")
    doc.preamble.append(NoEscape(r"\usepackage{" + nips_style + "}"))
    doc.preamble.append(NoEscape(r"\usepackage{setspace}"))
    doc.packages.append(Package("graphicx"))
    doc.packages.append(Package("enumitem"))

    # Adding a section without numbering
    with doc.create(Section("Feature importance with included context and text", numbering=False)):

        # Adding a subsection
        with doc.create(Subsection("Description of the method", numbering=False)):

            doc.append(bold("Feature importance "))
            doc.append(
                "method assigns a score to each feature. Features with a higher absolute score are considered more important in predicting individual cases. The sign of the score indicates whether the feature value contributes to or argues against the prediction. The SHAP method is one of the methods of local feature importance. \n \n"
            )
            doc.append(
                bold(
                    f"The scores are visualized relative to the average prediction, which is {round(starting_value, 2)}.\n"
                )
            )
            doc.append(
                "The final prediction is equal to the sum of the average prediction and the influence of all features.\n"
            )

        # Adding another subsection
        with doc.create(
            Subsection(
                f"Predicted probability of {target1} is {change[len(change)-1]:.2f}.",
                numbering=False,
            )
        ):
            with doc.create(Figure(position="h!")) as img_:
                img_.add_image("../plots/shap.pdf", width="400px")

    doc.append(bold("\nTextual explanation: "))
    doc.append(NoEscape(output))

    # Applying the function and printing the results
    results = check_values(
        feature_values,
        optimal_values,
        below_optimal_descriptions,
        optimal_descriptions,
        above_optimal_descriptions,
    )

    doc.append(bold("\nDomain context:"))

    with doc.create(Itemize(options=["noitemsep"])) as itemize:
        for i in range(3):
            feature = sorted_feature_names[i]
            value = results[feature_names.index(feature)]
            itemize.add_item(NoEscape(f"\\textbf{{{feature}}}: {value}"))

    if not os.path.exists("./prototypes"):
        os.makedirs("./prototypes")

    doc.generate_pdf(
        os.path.join(f"./prototypes/output_{index}"),
        compiler="pdflatex",
        clean=True,
        clean_tex=True,
    )



import os

import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns

# Set visualization styles
sns.set_theme(style="white", font_scale=2.5)
sns.set_palette("deep")
sns.set_context("paper")

plt.rcParams["xtick.bottom"] = True
plt.rcParams["ytick.left"] = True
plt.rcParams["font.family"] = "serif"
plt.rcParams["axes.labelsize"] = 18
plt.rcParams["xtick.labelsize"] = 16
plt.rcParams["ytick.labelsize"] = 16
plt.rcParams["axes.facecolor"] = "w"
plt.rc("font", weight="bold")


def plot_shap(shap_values: list, feature_values: list, fig_path: str) -> None:
    """
    Plot SHAP values for a given instance.

    Parameters
    ----------
    shap_values : list
        SHAP values for the instance.
    feature_values : list
        Feature values for the instance.
    fig_path : str
        Path to save the plot.

    Returns
    -------
    None
    """

    for index, item in enumerate(shap_values):
        new = round(item, 2)

        if new == 0:
            new = item

        shap_values[index] = new

    # Sort features by absolute SHAP value
    sorted_indices = sorted(
        range(len(shap_values)), key=lambda k: abs(shap_values[k]), reverse=True
    )
    sorted_features = [feature_values[i] for i in sorted_indices]
    sorted_shap_values = [shap_values[i] for i in sorted_indices]

    limit = 8
    if len(sorted_shap_values) > limit:
        sorted_shap_values = np.append(sorted_shap_values[:limit], sum(sorted_shap_values[limit:]))
        sorted_features = sorted_features[:limit]
        sorted_features.append("Other")

    # Set up the plot
    fig, ax = plt.subplots(figsize=(12, 8))
    bars = sns.barplot(
        x=sorted_shap_values, y=sorted_features, color="gray", edgecolor="black", ax=ax
    )

    for i, (bar, val) in enumerate(zip(bars.patches, sorted_shap_values)):
        if abs(val) < 0.05:
            text_color = "black"
            ha_alignment = "right" if val < 0 else "left"
            offset = 0.005 if val > 0 else -0.005
        else:
            text_color = "white"
            ha_alignment = "left" if val < 0 else "right"
            offset = 0.005 if val < 0 else -0.005

        val_str = f"{val:+.2f}"
        ax.text(
            val + offset,
            i,
            r"\textbf{{{val_str}}}".format(val_str=val_str),
            va="center",
            ha=ha_alignment,
            color=text_color,
            fontsize=16,
            weight="bold",
        )

    # Customize the plot
    ax.set_xlabel(r"\textbf{SHAP values}", fontsize=24, weight="bold")
    ax.set_ylabel(r"\textbf{Person characteristics}", fontsize=24, weight="bold")

    # Adding grid lines for better readability
    ax.grid(True, which="both", linestyle="--", linewidth=0.5, color="grey", axis="x")
    # Set line at 0 to be strong black
    ax.axvline(x=0, color="black", linewidth=1.5)

    # Enhancing the ticks
    ax.tick_params(axis="x", labelsize=20, width=1.5)
    ax.tick_params(axis="y", labelsize=20, width=1.5)

    # remove y axis ticks
    ax.yaxis.set_ticks_position("none")

    # set xlim to the range of the SHAP values
    ax.set_xlim(min(sorted_shap_values) - 0.05, max(sorted_shap_values) + 0.05)

    # Adding a border around the plot
    for spine in ["top", "right", "bottom", "left"]:
        ax.spines[spine].set_color("0.5")
        ax.spines[spine].set_visible(True)
    for spine in ["left", "bottom"]:
        ax.spines[spine].set_linewidth(1.2)

    fig.tight_layout()

    if os.path.exists(fig_path):
        os.remove(fig_path)
    else:
        os.makedirs(os.path.dirname(fig_path), exist_ok=True)

    fig.savefig(fig_path, format="pdf")



import pickle

import numpy as np
import pandas as pd
from sklearn.ensemble import GradientBoostingClassifier as XGBClassifier
from sklearn.metrics import f1_score
from sklearn.model_selection import train_test_split


def load_data(path: str) -> pd.DataFrame:
    return pd.read_csv(path)


def preprocess_data(df: pd.DataFrame) -> tuple:
    df["gender"] = df["gender"].astype("category").cat.codes
    df = df[df["smoking_history"] != "No Info"]
    smoker_status = ["current", "ever", "former"]
    df["smoking_history"] = df["smoking_history"].apply(lambda x: 1 if x in smoker_status else 0)

    X = df.drop(columns=["diabetes"])
    y = df["diabetes"]

    return X, y


def train_model(X: pd.DataFrame, y: pd.Series) -> XGBClassifier:
    model = XGBClassifier()
    model.fit(X, y)

    pickle.dump(model, open("./models/model.pkl", "wb"))
    return model


def predict(model: XGBClassifier, X: pd.DataFrame) -> np.ndarray:
    return model.predict(X)


if __name__ == "__main__":
    _is_trained = False
    data = load_data("kaggle_data/diabetes_prediction_dataset_balanced.csv")

    X, y = preprocess_data(data)

    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

    if not _is_trained:
        clf = train_model(X_train, y_train)
    else:
        clf = pickle.load(open("./models/diabetes_model.pkl", "rb"))

    y_pred = predict(clf, X_test)



from setuptools import find_packages, setup

VERSION = "1.0.9"
DESCRIPTION = "Contextualizing model's decisions with natural language explanations."
LONG_DESCRIPTION = """
# ConteXAI - Contextualizing model's decisions with natural language explanations

ConteXAI is a Python package developed as part of the Data Science Project Competition 2024. 

The library **links local explainability methods with natural language explanations**, allowing users to gain detailed insights into model predictions.
By incorporating additional domain knowledge, users can obtain even better insights into the model's predictions.

### Install

ConteXAI can be installed using pip:
```bash
pip install dscontexai
```

### Generate XAI reports

Using this library, you can fully automatically develop your XAI reports. Currently, it supports SHAP values, which is a local feature importance measure, to which we added textual explanations and domain-related context.
The library can be used for binary classification problems, for all models that output probabilities.

The report consists of four main segments: 

  (1) Description of local feature importances, 

  (2) Explanation of the instance's prediction, 

  (3) Visualization of local feature importances, 
  
  (4) Explanation of the visualization in natural language and context related to the domain and the prediction. 

<p align="center">
  <img src="https://raw.githubusercontent.com/jovanavidenovic/ConteXAI/main/other/XAI_2_page-0001.jpg" alt="Report structure" width="85%">
</p>

### Preparing configuration file
To use this tool with your model and dataset, you need to provide a configuration JSON file for a dataset, structured like the one below.

```bash
{
  # domain-based context
  "optimal_values": [
  ...
    [0, 6.5],               # HbA1c level optimal range is from 0 to 6.5
    [100, 140]              # Glucose level optimal range is from 100 to 140 
  ],
  "descriptions": {
    "below_optimal": [
      # Context for feature values falling below the optimal range
    ],
    "optimal": [
      # Context for feature values falling within the optimal range
    ],
    "above_optimal": [
      # Context for feature values falling above the optimal range
    ]
  },
  # Data transformations for feature mapping
  "transformation": {
    "0": ["Man", "Woman"],  # Transformation for Gender
    "1": [],                # No transformation for Age (numeric)
    "2": ["No", "Yes"],     # Transformation for Hypertension
    ...
  },
  "feature_names": [
    "Gender",
    "Age",
    "Hypertension",
    "Heart_disease",
    "Smoking_history",
    "BMI",
    "HbA1c_level",
    "Blood_glucose_level"
  ],
  # variables needed for structuring the description
  "target1": "diabetes",    # Target variable the model is predicting
  "target2": "Person",      # The object representing a sample in a data
  "supporting": ["does", "have"]  # Supporting verbs used in descriptions
}
```

#### Running the generation
Once you have model, dataset, and config file ready, to generate the report, you can use the following code snippet:
  
  ```python
  from dscontexai.generate_report import general

  general.generate_report(model_path= "path/to/model.pkl", dataset_path="path/to/data.csv", config="path/to/config.json", idx=sample_idx)
  ```

After successful generation, you will find a PDF report in the directory prototype/ under the name output_{sample_idx}.pdf.

#### Example notebooks

Working example, as well as the example of the generated report and the configuration file, can be found [here](https://github.com/jovanavidenovic/ConteXAI/tree/main/titanic).
 
#### Examples
Examples of the generated reports are shown below. Problem domain was diabetes prediction.

![Example 1](https://raw.githubusercontent.com/jovanavidenovic/ConteXAI/main/other/output_10_page-0001.jpg)
![Example 2](https://raw.githubusercontent.com/jovanavidenovic/ConteXAI/main/other/output_4130_page-0001.jpg)
"""

# Setting up
setup(
    name="dscontexai",
    version=VERSION,
    author="Jovana V., Haris K., Luka M.",
    author_email="hk8302@student.uni-lj.si",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    package_data={"dscontexai": ["latex/neurips_2023.sty"]},
    install_requires=[
        "asttokens==2.4.1",
        "backcall==0.2.0",
        "catboost==1.2.5",
        "cloudpickle==3.0.0",
        "comm==0.2.2",
        "contourpy==1.1.1",
        "cycler==0.12.1",
        "debugpy==1.8.1",
        "decorator==5.1.1",
        "executing==2.0.1",
        "fonttools==4.51.0",
        "graphviz==0.20.3",
        "importlib-metadata==7.1.0",
        "importlib-resources==6.4.0",
        "ipykernel==6.29.4",
        "ipython==8.12.3",
        "jedi==0.19.1",
        "joblib==1.4.0",
        "jupyter-client==8.6.1",
        "jupyter-core==5.7.2",
        "kiwisolver==1.4.5",
        "llvmlite==0.41.1",
        "matplotlib==3.7.5",
        "matplotlib-inline==0.1.7",
        "nest-asyncio==1.6.0",
        "numba==0.58.1",
        "numpy==1.24.4",
        "ordered-set==4.1.0",
        "packaging==24.0",
        "pandas==2.0.3",
        "parso==0.8.4",
        "pexpect==4.9.0",
        "pickleshare==0.7.5",
        "pillow==10.3.0",
        "platformdirs==4.2.0",
        "plotly==5.22.0",
        "prompt-toolkit==3.0.43",
        "psutil==5.9.8",
        "ptyprocess==0.7.0",
        "pure-eval==0.2.2",
        "pygments==2.17.2",
        "PyLaTeX==1.4.2",
        "pyparsing==3.1.2",
        "python-dateutil==2.9.0.post0",
        "pytz==2024.1",
        "pyzmq==26.0.0",
        "scikit-learn==1.3.2",
        "scipy==1.10.1",
        "seaborn==0.13.2",
        "shap==0.44.1",
        "six==1.16.0",
        "slicer==0.0.7",
        "stack-data==0.6.3",
        "tenacity==8.3.0",
        "threadpoolctl==3.4.0",
        "tornado==6.4",
        "tqdm==4.66.2",
        "traitlets==5.14.2",
        "typing-extensions==4.11.0",
        "tzdata==2024.1",
        "xgboost==2.0.3",
        "wcwidth==0.2.13",
        "zipp==3.18.1",
    ],
    long_description_content_type="text/markdown",
    keywords=["python", "first package"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)
